@@ -12,7 +12,7 @@ sub discovery {
                         oid => '.1.3.6.1.4.1.714.1.2.5.6.1.2.1.6.1',
                         up  => 1,
                      });
-      if ($description_new ne "null") {
+      if (($description_new ne "null") && ($description_new ne "noSuchObject")) {
 
          $description_new =~ s/^"//;
          $description_new =~ s/"$//;
@@ -26,4 +26,4 @@ sub discovery {
    return $description;
 }
 
-1;
\ No newline at end of file
+1;
